function param = analyse_RS(tips, times, alldata, pxcm)
% analyse_RS: finds model parameters from the merged tips
% clusters into three root types: tap roots, short laterals, long laterals
%
% tips              structure describing the individual roots
% times             time points of the measurement
% alldata           list of original outputs from RSA_GUI
% pxcm              pixel per centimeter
%
% param             root system architecture model parameters
%
% See also: analyse_RSP_app
%
% Copyright 2013 Daniel Leitner & Andrea Schnepf. See license.txt for details.
%

%%
% Type 1: Order 0
%
ind0 = find([tips.order]==0);
p0 = struct('r',0,'lb',0,'la',0,'ln',0,'nob',0,'a',0,'theta',0,'tropism',...
    [],'successor',[]);

% nob, lb, ln, la
[all_nob, all_lb, all_ln, all_la] = getZones(tips,ind0);
p0.nob = [mean(all_nob), std(all_nob)];  % observed number of branches, NOT max. number!!!
p0.lb = [mean(all_lb), std(all_lb)];
p0.ln = [mean(all_ln), std(all_ln)];
p0.la = [mean(all_la), std(all_la)]; % rhizotron boundaries produce artefacts

% r
[r,k] = getR(tips,times,ind0);
p0.r = r; %[r,0];
k0 = k; %[k,0];

% a
a = zeros(length(ind0),length(times));
for i = 1 : length(ind0)
    a(i,:) = tips(ind0(i)).area./[tips(ind0(i)).length];    
end
a=a(:);
a = a(~isnan(a));
a = a(~isinf(a));
p0.a = [mean(a)/2, std(a)/2];

% theta
p0.theta = [0,0]; 

% tropism 
sigma=getSigma(tips,ind0,alldata,pxcm);
p0.tropism = [1,0,sigma];

%%
% Order >= 1 (Cluster into short and long roots)
%
ind1 = find([tips.order]>=1);
p1l = struct('r',0,'lb',0,'la',0,'ln',0,'nob',0,'a',0,'theta',0,'tropism',...
    [],'successor',[]); % long
p1s = struct('r',0,'lb',0,'la',0,'ln',0,'nob',0,'a',0,'theta',0,'tropism',...
    [],'successor',[]); % short

dl=size(length(ind1),1); l=size(length(ind1),1);
for i = 1 : length(ind1)
    dl(i) = tips(ind1(i)).length(end) - tips(ind1(i)).length(end-1);
    l(i) = tips(ind1(i)).length(end);
end
cond = l>0.1; % minimum length in cm
l = l(cond); 
ind1(~cond)=[]; % delete rest

% Cluster analysis
[idx,c] = kmeans(l,2,'start',[0;27]); % only per length

%%
% Type 2: Short root
%
inds1 = ind1(idx==1); % indices of short order 1 roots

% nob, lb, ln, la
[all_nob, all_lb, all_ln, all_la] = getZones(tips,inds1);
p1s.nob = [mean(all_nob), std(all_nob)];
p1s.lb = [mean(all_lb), std(all_lb)];
p1s.ln = [mean(all_ln), std(all_ln)];
p1s.la = [mean(all_la), std(all_la)]; % rhizotron boundaries produce artefacts

% r
[r,k] = getR(tips,times,inds1);
p1s.r = r; 
k1s = k; 

% a
a = zeros(length(inds1),length(times));
for i = 1 : length(inds1)
    a(i,:) = tips(inds1(i)).area./[tips(inds1(i)).length];    
end
a=a(:);
a = a(~isnan(a));
a = a(~isinf(a));
p1s.a = [mean(a)/2, std(a)/2];

% theta
[theta,thetas] = getTheta(tips,inds1,alldata,pxcm);
p1s.theta = [theta,thetas];

% tropism
sigma=getSigma(tips,inds1,alldata,pxcm);
p1s.tropism = [1,0,sigma];

%%
% Type 3: Long roots
%
indl1 = ind1(idx==2); % indices of long order 1 roots

% nob, lb, ln, la
[all_nob, all_lb, all_ln, all_la] = getZones(tips,indl1);
p1l.ln = [mean(all_ln), std(all_ln)];
p1l.nob = [mean(all_nob), std(all_nob)];
p1l.la = [mean(all_la), std(all_la)]; % rhizotron boundaries produce artefacts
p1l.lb = [mean(all_lb), std(all_lb)];

% r
[r,k] = getR(tips,times,indl1);
p1l.r = r; 
k1l = k; 

% a
a = zeros(length(indl1),length(times));
for i = 1 : length(indl1)
    a(i,:) = tips(indl1(i)).area./[tips(indl1(i)).length];    
end
a=a(:);
a = a(~isnan(a));
a = a(~isinf(a));
p1l.a = [mean(a)/2, std(a)/2];

% theta
[theta,thetas] = getTheta(tips,indl1,alldata,pxcm);
p1l.theta = [theta,thetas];

% tropism
sigma=getSigma(tips,indl1,alldata,pxcm);
p1l.tropism = [1,0,sigma];


%%
% Connectivity
%

% abuse order field for root type index
for i = 1 : length(ind0), tips(ind0(i)).order=1; end
for i = 1 : length(indl1), tips(indl1(i)).order=2; end
for i = 1 : length(inds1), tips(inds1(i)).order=3; end

num = zeros(3,3);
for i = 1 : length(ind1)
   j = find(tips(ind1(i)).predecessor == [tips.number]);
   num(tips(j).order,tips(ind1(i)).order)=...
       num(tips(j).order,tips(ind1(i)).order)+1;
end
for i = 1 : 3 % calc probability
    num(i,:) = num(i,:)/sum(num(i,:));
end
p0.successor = [2, num(1,2); 3, num(1,3)];
p1l.successor = [2, num(2,2); 3, num(2,3)];
p1s.successor = [2, num(3,2); 3, num(3,3)];

p0.nob = (k0(1)-p0.la(1)-p0.lb(1))/p0.ln(1)+1;
p1l.nob = (k1l(1)-p1l.la(1)-p1l.lb(1))/p1l.ln(1)+1;
p1s.nob = (k1s(1)-p1s.la(1)-p1s.lb(1))/p1s.ln(1)+1;

% disp(k0);
% disp(k1l);
% disp(k1s);

param = {p0,p1l,p1s};





function [nob, lb, ln, la] = getZones(tips,ind)
% getZones: prepares nob, lb, ln and la
%

nob= zeros(length(ind),1); 
ln = []; 
la = zeros(length(ind),1); 
lb = zeros(length(ind),1);
% la = [];
% lb = [];

for i = 1 : length(ind)
    n = tips(ind(i)).number;
    successors = [tips.predecessor]==n;
    pl = full([tips(successors).prelength]); % branching node postions
    nob(i) = length(pl);
    if ~isempty(pl) % in case there is one or more branches
        pl = sort(pl); % sort positions
        la(i) = tips(ind(i)).length(end) - pl(end);
        lb(i) = pl(1);
%         la = [la, tips(ind(i)).length(end) - pl(end)];
%         lb = [lb pl(1)];
        ln = [ln, diff(pl)];
    else % in case there is no bracnch
            
        la(i) = 2*tips(ind(i)).length(end)/2; 
        lb(i) = tips(ind(i)).length(end)/2;
    
    end    
end



function [r,k] = getR(tips, times, ind)
% getR: approximates growth rate r and maximal length by fitting for 
% negative exponential growth function
%

t_=[]; l_=[];
for i = 1 : length(ind)    
    tip = tips(ind(i));
    
    lt = times - tip.ct; % local time
    t = lt(lt>0);
    l = tip.length(lt>0); % should cut away zeros    
    if l(1)==0
        disp('?');
    end
    t_= [t_,t];
    l_= [l_,l]; 
end

% find k and r 
fun1 = @(p) p(1).*(1-exp(-p(2)/p(1)*t_));
fun2 = @(p) sum((fun1(p)-l_).^2);
p = fminsearch(fun2, [5, 20]); % find paramters r and k by minimization

% find standard errors of k and r
dof=length(l_)-length(p); % degrees of freedom
sdr = sqrt(sum((l_ - fun1(p)).^2)/dof); % standard deviation of the residuals
dfdk = @(p) (1-exp(-p(2)./p(1).*t_))+p(1).*(-t_).*p(2)./(p(1)^2).*exp(-p(2)/p(1).*t_);
dfdr = @(p) t_.*exp(-p(2)/p(1).*t_);
J = [dfdk(p)' dfdr(p)']; % jacobian
CVM = sdr^2*inv(J'*J); % covariance matrix 
se = sqrt(diag(CVM))'; % Parameter standard errors
k=[p(1),se(1)];
r=[p(2),se(2)];

persistent C;
if isempty(C)
    C=1;
else
    C=C+1;
end
cc=[1,3,2];
subplot(1,length(times),cc(C));
lambda = @(t) k(1)*(1-exp((-r(1)/k(1))*t));
tt = linspace(0,times(end),1000);

hold on;
plot(t_,l_,'b*');
plot(tt,lambda(tt),'k','Linewidth',2);

ylabel('Length (cm)');
xlabel('Age (days)');
str = {'Tap root', 'Short laterals', 'Long laterals'};
title(str{C});

      


function [theta, thetas] = getTheta(tips,ind,d,pxcm)
% getTheta: returns the branching angles
%

Slen = cell(4,1); % calculate length of edges
for i = 1 : length(d)
    Slen{i} = getWeights(d{i}.A,d{i}.edges,0,'length');
end

t=[];
for i = 1 : length(ind)
    
    p1 = tips(ind(i)).path;
    ds = tips(ind(i)).dataset;
    ip = find(tips(ind(i)).predecessor==[tips.number]);
    
    if ~isempty(ip)
        R = d{ds}.angleradius*pxcm(ds)/10;% in pixel
        p2_ = tips(ip).path;
        ei = find(p2_==p1(1));
        if ei>1
            p2 = p2_(1:ei);
            v1 = getDirection(d{ds}.A, Slen{ds}, d{ds}.nodes, d{ds}.edges, p1, R);
            v2 = getDirection(d{ds}.A, Slen{ds}, d{ds}.nodes, d{ds}.edges, p2, R);
            t = [t, acos(v1*v2')];
        end
    end
    
end 
theta = mean(t);
thetas = std(t);



function sigma = getSigma(tips,ind,d,pxcm)
% getSigma:
%

Slen = cell(4,1); % calculate length of edges
for i = 1 : length(d)
    Slen{i} = getWeights(d{i}.A,d{i}.edges,0,'length');
end

N=0;
sigma=0;
c2=1;
c=1;
for i=1:length(ind)
    t=tips(ind(i));
    ds=t.dataset;
    p = getPathPoints(d{ds}.A, Slen{ds}, d{ds}.nodes, d{ds}.edges, t.path, inf);
    sl = 20;   %step length
    old = p(1,:);
    N=N+length(1+sl:sl:length(p)-sl);
    c3=1;
    for j=1+sl:sl:length(p)-sl
       v1=p(j,:)-old;
       v2=p(j+sl,:)-p(j,:);
       a=acos((v1/norm(v1))*(v2/norm(v2))');
       old=p(j,:);
       sigma=sigma+a^2;
       v1a(c3) = acos((v1/norm(v1))*[0;1]);
       v2a(c3) = acos((v2/norm(v2))*[0;1]); 
       c3=c3+1;
       a_(c)=sign(det([v1', v2']))*a;
       c=c+1; 
    end
    if (size(p,1)>=1+2*sl)
       mean_a(c2)=mean(a_);
       std_a(c2)=std(a_);
       starta(c2)=v1a(1);
       enda(c2)=v2a(end);
       c2=c2+1;
       clear v1a v2a;
    end   
end
sigma=sqrt(sigma/N)/sqrt(sl/pxcm(1));

% figure; hold on;
% %plot(v1a,v2a,'g*'); hold on
% 
% plot(linspace(0,pi,2),linspace(0,pi,2),'k');
% plot(starta,enda,'g*');
% xlabel('start'); ylabel('end');

% figure
%  hist(mean_a);



