function tips= mergeTips(dTips,times,pxcm)
% mergeTips: merge the dynamic measurements
%
% dTips             list of tip structures for each measurement
% times             time points of the measurement
% pxcm              pixel per centimeter
%
% tips              single structure containing dynamic information
%
% adds a field ct, which is the estimated creation time of the base root 
% adds a field r for linear growth rate between each measurements (mostly exact, in some cases relies on ct)
%
% Copyright 2013 Daniel Leitner & Andrea Schnepf. See license.txt for details.
%

N = length(dTips); % number of measurements

%
% merge length and area into tips
%
for i = 1 : length(dTips{N}) % number of roots
    
    t = dTips{N}(i);  
    tips(i) =  struct('ct',0,'length',zeros(1,N),'r',zeros(1,N),'path',t.path,...
        'predecessor',t.predecessor,'prelength',t.prelength/pxcm,'number',t.number,...
        'area',zeros(1,N),'order',t.order, 'la',0);    
    tips(i).length(N) = t.length/pxcm;
    tips(i).area(N) = t.area/pxcm^2;
    
    n = N;
    while n>1        
        n = n-1;
        ind = find(t.number == [dTips{n}.number]);
        if isempty(ind)
            break;
        else
            t = dTips{n}(ind);
            tips(i).length(n) = t.length/pxcm;
            tips(i).area(n) = t.area/pxcm^2;
        end
    end   
end

%
% set ct and r
%

% Order 0
dt = diff([0,times]);
ind =  find([tips.order]==0);
for i = 1 : length(ind)
    t = tips(ind(i));
    dl = diff([0,t.length]);
    tips(ind(i)).ct = 0;
    tips(ind(i)).r = dl./dt;    
    tips(ind(i)).la = dl(end)./dt(end) * 0.5; % apical zone half a day of growth
end


% Order >=1
for k = 1 : max([tips.order])

   ind = find([tips.order]==k); 
    
   for i = 1 : length(ind)
       
       t = tips(ind(i));
       ip = find([tips.number]==t.predecessor,1,'first'); % index of predecessor
       
       % creation time
       ct = getAge(tips(ip),t.prelength+tips(ip).la) + tips(ip).ct; 
       mint = times(find(t.length==0,1,'last'));      
       if isempty(mint)
           tips(ind(i)).ct=ct;
       else
           if ct<mint
               fprintf('*');
               tips(ind(i)).ct=mint;
           else
               tips(ind(i)).ct=ct;
           end
       end       
       
       % growth rate
       t_ = [0, times];
       l_ = [0, t.length];
       t_(find(l_==0,1,'last')) = tips(ind(i)).ct; % in some cases us ct
       
       dt = diff(t_);
       dl = diff(l_);
       tips(ind(i)).r = dl./dt;
       
       tips(ind(i)).la = dl(end)./dt(end) * 0.5; % apical zone half a day of growth
       
   end
   
   fprintf('\n');
   
end
   


function age=getAge(tip, L)
% getAge: age of root at length L, assuming piecewise constant growth
% 

age = 0;
dl = diff([0,tip.length]);

if L==0
    age=0;
    return
end

for i = 1 : length(dl) % measurements
    if dl(i)>0
        if L>dl(i)
            age = age+dl(i)/tip.r(i);
            L = L - dl(i);
        else
            age = age + L/tip.r(i);
            return
        end
    end    
end

disp('prelength > last measured length');




